using System;
using System.Collections.Concurrent;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using Newtonsoft.Json;
using WingDeviceService.Common;
using WingInterfaceLibrary.DTO.LiveRoom;
using WingInterfaceLibrary.Enum;
using WingInterfaceLibrary.Interface;
using WingInterfaceLibrary.Internal.Request;
using WingInterfaceLibrary.Notifications;
using WingInterfaceLibrary.Notifications.Live;
using WingInterfaceLibrary.Notifications.Remote;
using WingInterfaceLibrary.Request;
using WingInterfaceLibrary.Request.Authentication;
using WingInterfaceLibrary.Request.Device;
using WingInterfaceLibrary.Request.Remote;
using WingServerCommon.Interfaces.Cache;
using WingServerCommon.Service;
using WingInterfaceLibrary.Result.Device;
using WingServerCommon.Log;
using WingInterfaceLibrary.DTO.RemoteParameters;
using WingServerCommon.Mapper;
using WingServerCommon.Config;
using WingInterfaceLibrary.Enum.NotificationEnum;
using System.Runtime.CompilerServices;

namespace WingDeviceService.Service
{
    /// <summary>
    /// 远程管理
    /// </summary>
    public partial class DeviceService : JsonRpcService, IDeviceService
    {
        private ConcurrentDictionary<string, string> _remoteMaintainMsgs = new ConcurrentDictionary<string, string>();
        //探头缓存
        private ConcurrentDictionary<string, CacheControllingParameter> _cacheProbeApplicationSetting = new ConcurrentDictionary<string, CacheControllingParameter>();
        #region Remote Maintain
        /// <summary>
        /// 检查要连接的超声设备是否空闲
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> CheckDeviceIsIdleAsync(GetDeviceRequest request)
        {
            if (string.IsNullOrEmpty(request.DeviceCode))
            {
                ThrowCustomerException(CustomerRpcCode.NoExistDevice, "Device does not exist");
            }
            //验证是否有人已经连接这个超声机
            var result = _remoteConnectManager.IsIdle(request.Token, request.DeviceCode, TransactionTypeEnum.Consultion) == 0 ? true : false;
            return result;
        }

        /// <summary>
        /// 添加用户远程连接
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> AddUserRemoteConnectAsync(AddUserRemoteConnectRequest request)
        {
            try
            {
                if (string.IsNullOrEmpty(request.DeviceCode))
                {
                    ThrowCustomerException(CustomerRpcCode.NoExistDevice, "Device does not exist");
                }
                //验证是否有人已经连接这个超声机
                var key = request.Token + "_" + request.DeviceCode;
                var result = await _authenticationService.GetTokenAsync(new TokenRequest() { Token = request.Token }) ?? new TokenDTO();
                if (string.IsNullOrEmpty(result.ClientId))
                {
                    ThrowCustomerException(CustomerRpcCode.UsercodeIsEmpty, "User code is empty");
                }
                var userCode = result.ClientId;
                var req = new GetTokensWithClientIdRequest()
                {
                    ClientId = request.DeviceCode
                };
                var tokenList = _authenticationService.GetTokensWithClientIdAsync(req).Result ?? new List<TokenDTO>();
                var tokenInfo = tokenList?.Where(c => c.IsOnline)?.OrderByDescending(c => c.CreateTime)?.FirstOrDefault() ?? new TokenDTO();
                var deviceToken = tokenInfo.Code;
                if (result.LoginServer == ConfigurationManager.Host)
                {
                    _remoteConnectManager.AddOrUpdateHeart(key, true, request.Token, deviceToken, request.DeviceCode, userCode, request.RoomId, request.StatusEnum, result.LoginSource, TransactionTypeEnum.Consultion);
                }
                else
                {
                    //区分本服务器和不是本服务器
                    _remoteConnectManager.AddOrUpdate(key, request.Token, deviceToken, request.DeviceCode, userCode, request.RoomId, request.StatusEnum, result.LoginSource, TransactionTypeEnum.Consultion);
                }
                if (request.IsNeedSyn)
                {
                    var syncData = new RemoteConnectDataRequest()
                    {
                        RoomCode = request.RoomId,
                        UserRemoteConnect = _remoteConnectManager.GetRemoteConnectDTO(key).MappingTo<UserRemoteConnectRequest>()
                    };
                    if (syncData.UserRemoteConnect.StatusEnum == ConnectStatusEnum.CompleteConnect)
                    {
                        await SyncToMasterAsync(SyncTypeEnum.ConsultionDisConnect, request.RoomId, syncData);
                    }
                    else
                    {
                        await SyncToMasterAsync(SyncTypeEnum.ConsultionRemoteConnect, request.RoomId, syncData);
                    }
                }
                return true;
            }
            catch (Exception ex)
            {
                Logger.WriteLineWarn($"DeviceService AddUserRemoteConnectAsync error, userToken: {request.Token}, deviceCode: {request.DeviceCode}, errMsg: {ex}");
                return false;
            }
        }

        /// <summary>
        /// 检查用户是已经远程连接
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<string> CheckUserIsExistRoomIdAsync(GetDeviceRequest request)
        {
            //验证是否有人已经连接这个超声机
            if (string.IsNullOrEmpty(request.DeviceCode))
            {
                ThrowCustomerException(CustomerRpcCode.NoExistDevice, "Device does not exist");
            }
            var key = request.Token + "_" + request.DeviceCode;
            var result = _remoteConnectManager.GetRoomId(key);
            return result;
        }

        /// <summary>
        /// 关闭用户正在进行的远程连接
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> CloseUserRemoteConnectAsync(GetDeviceRequest request)
        {
            //关闭正在进行的链接
            if (string.IsNullOrEmpty(request.DeviceCode))
            {
                ThrowCustomerException(CustomerRpcCode.NoExistDevice, "Device does not exist");
            }
            var key = request.Token + "_" + request.DeviceCode;
            var syncData = new RemoteConnectDataRequest()
            {
                RoomCode = "",
                UserRemoteConnect = _remoteConnectManager.GetRemoteConnectDTO(key).MappingTo<UserRemoteConnectRequest>()
            };
            var result = _remoteConnectManager.Close(key);
            if (request.IsNeedSyn)
            {
                await SyncToMasterAsync(SyncTypeEnum.ConsultionDisConnect, "", syncData);
            }
            return result;
        }

        /// <summary>
        /// 会诊申请连接/断开请求接口
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> ConsultionApplyRemoteConnect(ControlDeviceConnectRequest request)
        {
            Logger.WriteLineWarn($"DeviceService ConsultionApplyRemoteConnect Init request: {JsonConvert.SerializeObject(request)}");
            bool res = false;
            var req = new TokenRequest() { Token = request.Token };
            var result = await _authenticationService.GetTokenAsync(req) ?? new TokenDTO();
            if (string.IsNullOrEmpty(request.DeviceCode))
            {
                ThrowCustomerException(CustomerRpcCode.NoExistDevice, "Device does not exist");
            }
            if (string.IsNullOrEmpty(result.ClientId))
            {
                ThrowCustomerException(CustomerRpcCode.UsercodeIsEmpty, "User code is empty");
            }
            //判断是否是会诊
            if (!string.IsNullOrEmpty(request.RoomCode))
            {
                var liveParameterReq = new WingInterfaceLibrary.LiveConsultation.ControlDeviceParameterInConsultationRequest()
                {
                    Token = request.Token,
                    ConsultationCode = request.RoomCode,
                    ControlType = request.ControlType,
                    IsNeedSyn = request.IsNeedSyn
                };
                res = await _liveConsultationService.ControlParameterAsync(liveParameterReq);
            }
            else
            {
                Logger.WriteLineWarn($"DeviceService ConsultionApplyRemoteConnect RoomCode Error: {request.RoomCode}");
            }
            Logger.WriteLineWarn($"DeviceService ConsultionApplyRemoteConnect Last res: {res}");
            return res;
        }

        /// <summary>
        /// 会诊设备端接受连接/断开链接请求接口
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> ConsultionAcceptRemoteConnnectionAsync(ControlDeviceResponseRequest request)
        {
            Logger.WriteLineWarn($"DeviceService ConsultionAcceptRemoteConnnectionAsync Init request: {JsonConvert.SerializeObject(request)}");
            var res = false;
            var deviceToken = await _authenticationService.GetTokenAsync(new TokenRequest() { Token = request.Token }) ?? new TokenDTO();
            string deviceCode = deviceToken.ClientId;
            if (string.IsNullOrEmpty(request.UserCode))
            {
                ThrowCustomerException(CustomerRpcCode.UsercodeIsEmpty, "User code is empty");
            }
            if (string.IsNullOrEmpty(deviceCode))
            {
                ThrowCustomerException(CustomerRpcCode.DeviceCodeEmpty, "Device code is empty");
            }

            var liveParameterReq = new WingInterfaceLibrary.Request.Remote.ControlDeviceResponseRequest()
            {
                Token = request.Token,
                ControlType = request.ControlType,
                UserCode = request.UserCode,
                LoginSource = request.LoginSource,
                IsNeedSyn = request.IsNeedSyn
            };
            res = await _liveConsultationService.AcceptRemoteConnnectionAsync(liveParameterReq);
            Logger.WriteLineWarn($"DeviceService ConsultionAcceptRemoteConnnectionAsync Last res: {res}");
            return res;
        }

        /// <summary>
        /// 申请连接/断开请求接口
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> ApplyRemoteConnectionAsync(ControlDeviceConnectRequest request)
        {
            Logger.WriteLineWarn($"DeviceService ApplyRemoteConnectionAsync Init request: {JsonConvert.SerializeObject(request)}");

            var req = new TokenRequest() { Token = request.Token };
            var result = await _authenticationService.GetTokenAsync(req) ?? new TokenDTO();
            if (string.IsNullOrEmpty(request.DeviceCode))
            {
                ThrowCustomerException(CustomerRpcCode.NoExistDevice, "Device does not exist");
            }
            if (string.IsNullOrEmpty(result.ClientId))
            {
                ThrowCustomerException(CustomerRpcCode.UsercodeIsEmpty, "User code is empty");
            }
            var deviceCode = request.DeviceCode;
            var userCode = result.ClientId;
            //获取用户信息和设备信息
            var reqDevice = new GetTokensWithClientIdRequest()
            {
                ClientId = deviceCode
            };
            var tokenList = _authenticationService.GetTokensWithClientIdAsync(reqDevice).Result ?? new List<TokenDTO>();
            var deviceTokenInfo = tokenList?.Where(c => c.IsOnline)?.OrderByDescending(c => c.CreateTime)?.FirstOrDefault() ?? new TokenDTO();
            var deviceToken = deviceTokenInfo.Code;
            var isOldPlatform = deviceTokenInfo != null && deviceTokenInfo.IsOldPlatform && deviceTokenInfo.IsOnline;

            //判断是否是会诊
            if (!string.IsNullOrEmpty(request.RoomCode))
            {
                var liveParameterReq = new WingInterfaceLibrary.LiveConsultation.ControlDeviceParameterInConsultationRequest()
                {
                    Token = request.Token,
                    ConsultationCode = request.RoomCode,
                    ControlType = request.ControlType
                };
                var r = await _liveConsultationService.ControlParameterAsync(liveParameterReq);
                return r;
            }

            var roomId = result.ClientId + request.DeviceCode;
            var notification = new ConnectStatusToDeviceNotification()//设备通知
            {
                ControlUserCode = result.ClientId,
                ControlUserName = result.AccountName,
                ControlType = request.ControlType,
                TransactionType = TransactionTypeEnum.AfterSales,
                LoginSource = result.LoginSource,
                DeviceCode = request.DeviceCode,
            };
            BroadcastNotificationRequest notificationRequest = new BroadcastNotificationRequest();
            notificationRequest.ClientIds = new List<string>() { request.DeviceCode };
            notificationRequest.Message = notification;
            notificationRequest.JsonMessage = Newtonsoft.Json.JsonConvert.SerializeObject(notification);
            notificationRequest.NotificationType = notification.NotificationType;
            notificationRequest.TransactionType = TransactionTypeEnum.AfterSales;
            notificationRequest.RelevanceCode = roomId;
            notificationRequest.ReceiverType = ApplicantTypeEnum.Device;
            notificationRequest.WSConnectType = WSConnectTypeEnum.RemoteConnectSecondWindow;
            notificationRequest.IsNeedSyn = false;
            //客户端申请连接
            if (request.ControlType == ControlDeviceParameterEnum.Start)
            {
                //验证用户机构和设备机构是否同一个
                //var deviceInfo = await _deviceInfoDBServiceProxy.FindDeviceInfoByCodeAsync(request.DeviceCode) ?? new WingInterfaceLibrary.DTO.Device.DeviceInfoDTO();
                var userInfo = await _userDBService.FindUserDTOByCodeAsync(result.ClientId, false) ?? new WingInterfaceLibrary.DTO.User.UserDTO();
                if (userInfo.UserFeatureCodes != null && userInfo.UserFeatureCodes.Exists(c => c == "RemoteMaintain" || c == "RemoteManager"))
                {
                    //针对远程维护管理员，只要个人添加了设备，远程维护不做任何限制
                }
                else
                {
                    //机构管理员则要验证是否本机构设备，否则不能远程维护
                    ThrowCustomerException(CustomerRpcCode.UserAndDeviceNoSameOrganization, "User and device no same organization");
                    // if (string.IsNullOrEmpty(userInfo.OrganizationCode) || string.IsNullOrEmpty(deviceInfo.OrganizationCode) || userInfo.OrganizationCode != deviceInfo.OrganizationCode)
                    // {
                    //     ThrowCustomerException(CustomerRpcCode.UserAndDeviceNoSameOrganization, "User and device no same organization");
                    // }
                }
                //验证是否有人已经连接这个超声机
                var statusValue = _remoteConnectManager.IsIdle(request.Token, request.DeviceCode);
                if (statusValue == 1)
                {
                    Logger.WriteLineWarn($"DeviceService ApplyRemoteConnectionAsync error, userToken: {request.Token}, deviceCode: {request.DeviceCode}, clientId: {result.ClientId}, userName: {result.AccountName}");
                    ThrowCustomerException(CustomerRpcCode.RemoteConnectDeviceOccupied, "Remote connect device has been occupied");
                }
                else if (statusValue == 2)
                {
                    Logger.WriteLineWarn($"DeviceService ApplyRemoteConnectionAsync error, userToken: {request.Token}, deviceCode: {request.DeviceCode}, clientId: {result.ClientId}, userName: {result.AccountName}");
                    ThrowCustomerException(CustomerRpcCode.UserIsAreadyAnotherDeviceUseRemoteConnect, "The user is already on another device use remote Connect");
                }

                if (isOldPlatform)
                {
                    if (request.IsNeedSyn)
                    {
                        var applyRemoteConnectResult = await _deviceForwardService.ApplyRemoteConnectAsync(notification);
                        if (!applyRemoteConnectResult)
                        {
                            Logger.WriteLineWarn($"DeviceService ApplyRemoteConnectionAsync DeviceForwardService error, userToken: {request.Token}, deviceCode: {request.DeviceCode}, clientId: {result.ClientId}, userName: {result.AccountName}");
                            ThrowCustomerException(CustomerRpcCode.RemoteConnectDeviceOccupied, "Remote connect device has been occupied");
                        }
                    }
                }

                var key = request.Token + "_" + request.DeviceCode;
                //_remoteConnectManager.AddOrUpdate(key, request.Token, deviceToken, request.DeviceCode, result.ClientId, roomId, ConnectStatusEnum.WaitConnect, result.LoginSource);
                if (result.LoginServer == ConfigurationManager.Host)
                {
                    _remoteConnectManager.AddOrUpdateHeart(key, true, request.Token, deviceToken, request.DeviceCode, result.ClientId, roomId, ConnectStatusEnum.WaitConnect, result.LoginSource);
                }
                else
                {
                    //区分本服务器和不是本服务器
                    _remoteConnectManager.AddOrUpdate(key, request.Token, deviceToken, request.DeviceCode, result.ClientId, roomId, ConnectStatusEnum.WaitConnect, result.LoginSource);
                }

                if (request.IsNeedSyn)
                {
                    var syncData = new RemoteConnectDataRequest()
                    {
                        RoomCode = roomId,
                        UserRemoteConnect = _remoteConnectManager.GetRemoteConnectDTO(key).MappingTo<UserRemoteConnectRequest>()
                    };
                    await SyncToMasterAsync(SyncTypeEnum.ApplyRemoteConnect, roomId, syncData);
                }
            }
            else if (request.ControlType == ControlDeviceParameterEnum.EndRunExecuteApi)
            {
                //结束调参不需要做任何处理，这里要跟断开连接，区分开来
                var changeStateResult = await _rtcService.ChangeControllingStateAsync(new ChangeControllingStateRequest
                {
                    LiveRoomCode = roomId,
                    UserCode = result.ClientId,
                    IsControllingParameter = false,
                });
                return true;
            }
            //客户端申请断开
            else if (request.ControlType == ControlDeviceParameterEnum.End)
            {
                var key = request.Token + "_" + request.DeviceCode;
                var existRoomId = _remoteConnectManager.GetRoomId(key);
                if (string.IsNullOrEmpty(existRoomId))
                {
                    ThrowCustomerException(CustomerRpcCode.DeviceNotFind, "Device not find");
                }

                if (request.IsNeedSyn || result.LoginServer == ConfigurationManager.Host)
                {
                    _remoteConnectManager.AddOrUpdateHeart(key, true, request.Token, deviceToken, request.DeviceCode, result.ClientId, roomId, ConnectStatusEnum.WaitDisconnect, result.LoginSource);
                }
                else
                {
                    //区分本服务器和不是本服务器
                    _remoteConnectManager.AddOrUpdate(key, request.Token, deviceToken, request.DeviceCode, result.ClientId, roomId, ConnectStatusEnum.WaitDisconnect, result.LoginSource);
                }
                //数据同步
                if (request.IsNeedSyn)
                {
                    var syncData = new RemoteConnectDataRequest()
                    {
                        RoomCode = roomId,
                        UserRemoteConnect = _remoteConnectManager.GetRemoteConnectDTO(key).MappingTo<UserRemoteConnectRequest>()
                    };
                    await SyncToMasterAsync(SyncTypeEnum.ApplyDisConnect, roomId, syncData);
                }
            }
            if (isOldPlatform && request.ControlType != ControlDeviceParameterEnum.Start)
            {
                if (request.IsNeedSyn)
                {
                    await _deviceForwardService.ApplyRemoteConnectAsync(notification);
                }
            }
            var res = await _notificationService.BroadcastMessageAsync(notificationRequest);
            Logger.WriteLineWarn($"DeviceService ApplyRemoteConnectionAsync Last res: {res}");
            return res;
        }

        /// <summary>
        /// 断开远程连接，并通知客户端更新状态
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> DisconnectRemoteControl(RemoteConnectStautsRequest request)
        {
            Logger.WriteLineWarn($"DeviceService DisconnectRemoteControl Init request: {JsonConvert.SerializeObject(request)}");
            var result = false;
            var deviceToken = await _authenticationService.GetTokenAsync(new TokenRequest() { Token = request.Token }) ?? new TokenDTO();
            string deviceCode = await GetClientIdByTokenAsync(request.Token);
            if (string.IsNullOrEmpty(deviceCode) && !string.IsNullOrEmpty(request.DeviceCode))
            {
                deviceCode = request.DeviceCode;
            }
            if (string.IsNullOrEmpty(deviceCode))
            {
                ThrowCustomerException(CustomerRpcCode.NoExistDevice, "Device does not exist");
            }
            if (string.IsNullOrEmpty(request.UserCode))
            {
                //表示未连接任何用户，
            }
            else
            {
                //已连接，找到token，添加到缓存中
                var req = new GetTokensWithClientIdRequest()
                {
                    ClientId = request.UserCode,
                    AccountType = AccountType.User
                };
                var tokenList = await _authenticationService.GetTokensWithClientIdAsync(req) ?? new List<TokenDTO>();
                var tokenInfo = tokenList?.Where(c => c.IsOnline && c.LoginSource == request.LoginSource)?.OrderByDescending(c => c.CreateTime)?.FirstOrDefault() ?? new TokenDTO();
                var token = tokenInfo.Code;

                if (!string.IsNullOrEmpty(token))
                {
                    var key = token + "_" + deviceCode;
                    var roomId = request.UserCode + deviceCode;
                    var deviceKey = request.Token + "_" + request.UserCode;
                    var syncData = new RemoteConnectDataRequest()
                    {
                        RoomCode = roomId,
                        UserRemoteConnect = _remoteConnectManager.GetRemoteConnectDTO(key).MappingTo<UserRemoteConnectRequest>()
                    };
                    var closeManagerResult = _remoteConnectManager.Close(key);
                    var closeDeviceManagerResult = _remoteConnectManager.DeviceClose(deviceKey);
                    //断开/关闭连接，关闭房间
                    var liveRoom = await _rtcService.GetLiveRoomAsync(new GetLiveRoomRequest
                    {
                        LiveRoomCode = roomId
                    });
                    if (liveRoom != null)
                    {
                        var closeRTCResult = await _rtcService.CloseAsync(new CloseRequest
                        {
                            LiveRoomCode = roomId,
                            UserCode = request.UserCode,
                        });
                    }
                    if (request.IsNeedSyn)
                    {
                        await SyncToMasterAsync(SyncTypeEnum.DeviceDisconnect, roomId, syncData);
                    }
                    var deviceInfo = CacheMaintenance.Instance.Get<IDeviceInfosManager>().Where(c => c.Code == deviceCode).FirstOrDefault() ?? new WingServerCommon.Interfaces.Cache.CacheDeviceDTO();
                    var deviceName = deviceInfo.DisplayName;
                    //同步推送消息给客户端   
                    // var notification = new ConnectStatusToClientNotification()//客户端通知
                    // {
                    //     DeviceCode = deviceCode,
                    //     DeviceName = deviceName,
                    //     ControlType = ControlDeviceParameterEnum.End,
                    //     RemoteDeviceState = RemoteDeviceStateEnum.DeviceDisconnect
                    // };
                    var notification = new DeviceDisconnectRemoteControlNotification()
                    {
                        DeviceCode = deviceCode,
                        RemoteDeviceState = RemoteDeviceStateEnum.DeviceDisconnect
                    };
                    BroadcastNotificationRequest notificationRequest = new BroadcastNotificationRequest();
                    notificationRequest.ClientIds = new List<string>() { request.UserCode };
                    notificationRequest.Tokens = new List<TokenDTO>() { tokenInfo };
                    notificationRequest.Message = notification;
                    notificationRequest.JsonMessage = Newtonsoft.Json.JsonConvert.SerializeObject(notification);
                    notificationRequest.NotificationType = notification.NotificationType;
                    notificationRequest.TransactionType = TransactionTypeEnum.AfterSales;
                    notificationRequest.RelevanceCode = deviceCode;
                    notificationRequest.ReceiverType = ApplicantTypeEnum.Client;
                    notificationRequest.WSConnectType = WSConnectTypeEnum.RemoteConnectSecondWindow;
                    result = await _notificationService.BroadcastMessageAsync(notificationRequest);
                    Logger.WriteLineWarn($"DeviceService DisconnectRemoteControl Last res: {result}");
                }
                else
                {
                    ThrowCustomerException(CustomerRpcCode.UserIsAreadyAnotherDeviceUseRemoteConnect, "User login source request error");
                }
            }
            return result;
        }

        /// <summary>
        /// 设备端接受连接/断开链接请求接口
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> AcceptRemoteConnnectionAsync(ControlDeviceResponseRequest request)
        {
            Logger.WriteLineWarn($"DeviceService AcceptRemoteConnnectionAsync Init request: {JsonConvert.SerializeObject(request)}");
            var res = false;
            var deviceToken = await _authenticationService.GetTokenAsync(new TokenRequest() { Token = request.Token }) ?? new TokenDTO();
            string deviceCode = deviceToken.ClientId;
            if (string.IsNullOrEmpty(request.UserCode))
            {
                ThrowCustomerException(CustomerRpcCode.UsercodeIsEmpty, "User code is empty");
            }
            if (string.IsNullOrEmpty(deviceCode))
            {
                ThrowCustomerException(CustomerRpcCode.DeviceCodeEmpty, "Device code is empty");
            }
            var remoteConnectRooms = _remoteConnectManager.GetAllRemoteConnectList(deviceCode);
            if (!remoteConnectRooms.Any(x => x.LoginSource == request.LoginSource) && remoteConnectRooms.Any(x => x.StatusEnum == ConnectStatusEnum.WaitConnect))
            {
                request.LoginSource = remoteConnectRooms.FirstOrDefault(x => x.StatusEnum == ConnectStatusEnum.WaitConnect).LoginSource;
            }
            var tokenList = await _authenticationService.GetTokensWithClientIdAsync(new GetTokensWithClientIdRequest() { ClientId = request.UserCode }) ?? new List<TokenDTO>();
            var result = tokenList?.Where(c => c.IsOnline && c.LoginSource == request.LoginSource)?.OrderByDescending(c => c.CreateTime)?.FirstOrDefault() ?? new TokenDTO();
            if (string.IsNullOrEmpty(result.ClientId))
            {
                ThrowCustomerException(CustomerRpcCode.UsercodeIsEmpty, "User code is empty");
            }
            var key = result.Code + "_" + deviceCode;
            var deviceKey = request.Token + "_" + request.UserCode;
            var existRoomIdResult = _remoteConnectManager.GetRoomId(key);
            if (string.IsNullOrEmpty(existRoomIdResult))
            {
                Logger.WriteLineWarn($"AcceptRemoteConnnectionAsync RoomId Empty, key: {key}, userCode: {request.UserCode}, deviceToken: {request.Token}");
                ThrowCustomerException(CustomerRpcCode.UserNotConnected, "User not connected");
            }
            var controlUserToken = result.Code;
            //var result = await _authenticationService.GetTokenAsync(new TokenRequest() { Token = request.ControlUserToken }) ?? new TokenDTO();
            var deviceInfo = CacheMaintenance.Instance.Get<IDeviceInfosManager>().Where(c => c.Code == deviceCode).FirstOrDefault() ?? new WingServerCommon.Interfaces.Cache.CacheDeviceDTO();
            var roomId = request.UserCode + deviceCode;
            var deviceName = deviceInfo.DisplayName;
            //接受连接，创建房间
            if (request.ControlType == ControlDeviceParameterEnum.Start)
            {
                if (result.LoginServer == ConfigurationManager.Host)
                {
                    _remoteConnectManager.AddOrUpdateHeart(key, true, controlUserToken, request.Token, deviceCode, request.UserCode, roomId, ConnectStatusEnum.CompleteConnect, result.LoginSource);
                }
                else
                {
                    _remoteConnectManager.AddOrUpdate(key, controlUserToken, request.Token, deviceCode, request.UserCode, roomId, ConnectStatusEnum.CompleteConnect, request.LoginSource);
                }

                if (deviceToken.LoginServer == ConfigurationManager.Host)
                {
                    //device加入心跳
                    _remoteConnectManager.DeviceAddOrUpdateHeart(deviceKey, true, request.Token, controlUserToken, request.UserCode, deviceCode, TransactionTypeEnum.AfterSales);
                }
                else
                {
                    //device加入心跳
                    _remoteConnectManager.DeviceAddOrUpdate(deviceKey, request.Token, controlUserToken, request.UserCode, deviceCode, TransactionTypeEnum.AfterSales);
                }
                if (request.IsNeedSyn)
                {
                    //数据同步
                    var syncData = new RemoteConnectDataRequest()
                    {
                        RoomCode = roomId,
                        UserRemoteConnect = _remoteConnectManager.GetRemoteConnectDTO(key).MappingTo<UserRemoteConnectRequest>()
                    };
                    await SyncToMasterAsync(SyncTypeEnum.AcceptRemoteConnect, roomId, syncData);
                }
            }
            else if (request.ControlType == ControlDeviceParameterEnum.End)
            {
                var syncData = new RemoteConnectDataRequest()
                {
                    RoomCode = roomId,
                    UserRemoteConnect = _remoteConnectManager.GetRemoteConnectDTO(key).MappingTo<UserRemoteConnectRequest>()
                };
                var closeManagerResult = _remoteConnectManager.Close(key);
                var closeDeviceManagerResult = _remoteConnectManager.DeviceClose(deviceKey);
                if (request.IsNeedSyn)
                {
                    await SyncToMasterAsync(SyncTypeEnum.AcceptDisConnect, roomId, syncData);
                }
                var notify = new DeviceDisconnectRemoteControlNotification()
                {
                    DeviceCode = deviceCode,
                };
                var endNotificationRequest = new SendNotificationRequest()
                {
                    ClientId = request.UserCode,
                    Tokens = new List<TokenDTO>() { result },
                    Message = notify,
                    JsonMessage = Newtonsoft.Json.JsonConvert.SerializeObject(notify),
                    NotificationType = notify.NotificationType,
                    TransactionType = TransactionTypeEnum.AfterSales,
                    RelevanceCode = request.UserCode,
                    ReceiverType = ApplicantTypeEnum.Client,
                    IsNeedSyn = false,
                    WSConnectType = WSConnectTypeEnum.RemoteConnectSecondWindow
                };
                res = await _notificationService.PostMessageAsync(endNotificationRequest);
                Logger.WriteLineWarn($"DeviceService AcceptRemoteConnnectionAsync End res: {res}");
                return res;
            }
            else if (request.ControlType == ControlDeviceParameterEnum.RejectConnect)
            {
                var syncData = new RemoteConnectDataRequest()
                {
                    RoomCode = roomId,
                    UserRemoteConnect = _remoteConnectManager.GetRemoteConnectDTO(key).MappingTo<UserRemoteConnectRequest>()
                };
                var closeManagerResult = _remoteConnectManager.Close(key);
                if (request.IsNeedSyn)
                {
                    await SyncToMasterAsync(SyncTypeEnum.RejectRemoteConnect, roomId, syncData);
                }
                var notify = new DeviceRejectRemoteControlNotification()
                {
                    DeviceCode = deviceCode,
                };
                var rejectNotificationRequest = new SendNotificationRequest()
                {
                    ClientId = request.UserCode,
                    Tokens = new List<TokenDTO>() { result },
                    Message = notify,
                    JsonMessage = Newtonsoft.Json.JsonConvert.SerializeObject(notify),
                    NotificationType = notify.NotificationType,
                    TransactionType = TransactionTypeEnum.AfterSales,
                    RelevanceCode = request.UserCode,
                    ReceiverType = ApplicantTypeEnum.Client,
                    IsNeedSyn = false,
                    WSConnectType = WSConnectTypeEnum.RemoteConnectSecondWindow
                };
                res = await _notificationService.PostMessageAsync(rejectNotificationRequest);
                Logger.WriteLineWarn($"DeviceService AcceptRemoteConnnectionAsync RejectConnect res: {res}");
                return res;
            }
            var notification = new ConnectStatusToClientNotification()//客户端通知
            {
                DeviceCode = deviceCode,
                DeviceName = deviceName,
                ControlType = request.ControlType,
                RemoteDeviceState = RemoteDeviceStateEnum.Success
            };
            BroadcastNotificationRequest notificationRequest = new BroadcastNotificationRequest();
            notificationRequest.ClientIds = new List<string>() { request.UserCode };
            notificationRequest.Message = notification;
            notificationRequest.Tokens = new List<TokenDTO>() { result };
            notificationRequest.JsonMessage = Newtonsoft.Json.JsonConvert.SerializeObject(notification);
            notificationRequest.NotificationType = notification.NotificationType;
            notificationRequest.TransactionType = TransactionTypeEnum.AfterSales;
            notificationRequest.RelevanceCode = request.UserCode;
            notificationRequest.ReceiverType = ApplicantTypeEnum.Client;
            notificationRequest.IsNeedSyn = false;
            notificationRequest.WSConnectType = WSConnectTypeEnum.RemoteConnectSecondWindow;
            res = await _notificationService.BroadcastMessageAsync(notificationRequest);
            Logger.WriteLineWarn($"DeviceService AcceptRemoteConnnectionAsync Last res: {res}");
            return res;
        }

        /// <summary>
        /// 设备端远程维护连接心跳
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> DeviceRemoteConnectHeartRateAsync(RemoteConnectHeartRateRequest request)
        {
            try
            {
                Logger.WriteLineWarn($"DeviceService DeviceRemoteConnectHeartRateAsync init: {JsonConvert.SerializeObject(request)}");
                string deviceCode = await GetClientIdByTokenAsync(request.Token);
                if (string.IsNullOrEmpty(deviceCode))
                {
                    ThrowCustomerException(CustomerRpcCode.DeviceCodeEmpty, "Device code is empty");
                }

                var dataList = _remoteConnectManager.GetAllRemoteConnectByDeviceToken(request.Token);
                if (dataList?.Count > 0)
                {
                    foreach (var item in dataList)
                    {
                        var key = item.DeviceToken + "_" + item.UserCode;

                        var req = new GetTokensWithClientIdRequest()
                        {
                            ClientId = item.UserCode
                        };
                        var tokenList = _authenticationService.GetTokensWithClientIdAsync(req).Result ?? new List<TokenDTO>();
                        var tokenInfo = tokenList?.Where(c => c.IsOnline)?.OrderByDescending(c => c.CreateTime)?.FirstOrDefault() ?? new TokenDTO();
                        var userToken = tokenInfo.Code;

                        _remoteConnectManager.DeviceAddOrUpdateHeart(key, request.IsNeedSyn, item.DeviceToken, userToken, item.UserCode, deviceCode, request.TransactionType);
                        if (request.IsNeedSyn)
                        {
                            //数据同步
                            var syncData = new RemoteConnectDataRequest()
                            {
                                RoomCode = item.UserCode + deviceCode,
                                UserRemoteConnect = new UserRemoteConnectRequest()
                                {
                                    TransactionType = request.TransactionType,
                                    StatusEnum = ConnectStatusEnum.CompleteConnect,
                                    UserToken = userToken,
                                    DeviceToken = item.DeviceToken,
                                    UserCode = item.UserCode,
                                    RoomId = item.UserCode + deviceCode,
                                    DeviceCode = deviceCode
                                }
                            };
                            await SyncToMasterAsync(SyncTypeEnum.DeviceRemoteConnectHeartRate, syncData.RoomCode, syncData);
                        }
                    }
                    return true;
                }
                return false;
            }
            catch (Exception ex)
            {
                Logger.WriteLineWarn($"DeviceService DeviceRemoteConnectHeartRateAsync error, userToken: {request.Token}, errMsg: {ex}");
                return false;
            }
        }

        /// <summary>
        /// 远程维护连接心跳
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> RemoteConnectHeartRateAsync(RemoteConnectHeartRateRequest request)
        {
            try
            {
                Logger.WriteLineWarn($"DeviceService RemoteConnectHeartRateAsync init: {JsonConvert.SerializeObject(request)}");
                var result = await _authenticationService.GetTokenAsync(new TokenRequest() { Token = request.Token }) ?? new TokenDTO();
                if (string.IsNullOrEmpty(result.ClientId))
                {
                    ThrowCustomerException(CustomerRpcCode.UsercodeIsEmpty, "User code is empty");
                }
                string userCode = result.ClientId;
                var dataList = _remoteConnectManager.GetAllRemoteConnectByUserToken(request.Token);
                if (dataList?.Count > 0)
                {
                    foreach (var item in dataList)
                    {
                        var reqDevice = new GetTokensWithClientIdRequest()
                        {
                            ClientId = item.DeviceCode
                        };
                        var tokenList = _authenticationService.GetTokensWithClientIdAsync(reqDevice).Result ?? new List<TokenDTO>();
                        var tokenInfo = tokenList?.Where(c => c.IsOnline)?.OrderByDescending(c => c.CreateTime)?.FirstOrDefault() ?? new TokenDTO();
                        var deviceToken = tokenInfo.Code;

                        var roomId = item.RoomId;
                        var key = request.Token + "_" + item.DeviceCode;
                        _remoteConnectManager.AddOrUpdateHeart(key, request.IsNeedSyn, request.Token, deviceToken, item.DeviceCode, userCode, roomId, ConnectStatusEnum.CompleteConnect, result.LoginSource, request.TransactionType);

                        if (request.IsNeedSyn)
                        {
                            //数据同步
                            var syncData = new RemoteConnectDataRequest()
                            {
                                RoomCode = userCode + item.DeviceCode,
                                UserRemoteConnect = new UserRemoteConnectRequest()
                                {
                                    TransactionType = request.TransactionType,
                                    StatusEnum = ConnectStatusEnum.CompleteConnect,
                                    UserToken = request.Token,
                                    DeviceToken = deviceToken,
                                    UserCode = userCode,
                                    RoomId = userCode + item.DeviceCode,
                                    DeviceCode = item.DeviceCode
                                }
                            };
                            await SyncToMasterAsync(SyncTypeEnum.UserRemoteConnectHeartRate, syncData.RoomCode, syncData);
                        }
                    }
                    return true;
                }
                return false;
            }
            catch (Exception ex)
            {
                Logger.WriteLineWarn($"DeviceService RemoteConnectHeartRateAsync error, userToken: {request.Token}, errMsg: {ex}");
                return false;
            }
        }

        /// <summary>
        /// 远程连接设备不在线逻辑处理
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        private bool DeviceRemoteConnectOnlineState(DeviceRemoteConnectDTO deviceRemoteConnnectInfo, bool isNeedSyn, bool isManangeDisConnnect = false)
        {
            var res = false;
            var userCode = deviceRemoteConnnectInfo.UserCode;
            var userToken = deviceRemoteConnnectInfo.UserToken;
            var deviceCode = deviceRemoteConnnectInfo.DeviceCode;
            var deviceToken = deviceRemoteConnnectInfo.DeviceToken;
            var key = userToken + "_" + deviceCode;
            var deviceKey = deviceToken + "_" + userCode;
            var existRoomId = _remoteConnectManager.GetExistDeviceRoomId(deviceKey);
            Logger.WriteLineInfo($"DeviceService DeviceRemoteConnectOnlineState key-deviceKey:{key}-{deviceKey}, isNeedSyn: {isNeedSyn}");
            if (!existRoomId)
            {
                Logger.WriteLineWarn($"DeviceService DeviceRemoteConnectOnlineState RoomId Empty, key: {key}, isNeedSyn: {isNeedSyn}");
                return true;
            }
            var roomId = userCode + deviceCode;
            var syncData = new RemoteConnectDataRequest()
            {
                RoomCode = roomId,
                UserRemoteConnect = _remoteConnectManager.GetRemoteConnectDTO(key).MappingTo<UserRemoteConnectRequest>(),
                IsManangeDisConnnect = isManangeDisConnnect
            };
            var closeManagerResult = _remoteConnectManager.Close(key);
            var closeDeviceManagerResult = _remoteConnectManager.DeviceClose(deviceKey);
            if (isNeedSyn)
            {
                SyncToMasterAsync(SyncTypeEnum.DeviceRemoteConnectOutline, roomId, syncData).GetAwaiter().GetResult();
            }
            var tokenInfo = _authenticationService.GetTokenAsync(new TokenRequest() { Token = userToken }).Result;
            var notify = new DeviceDisconnectRemoteControlNotification()
            {
                DeviceCode = deviceCode,
                RemoteDeviceState = isManangeDisConnnect ? RemoteDeviceStateEnum.DeviceDisconnect : RemoteDeviceStateEnum.DeviceTimeOutDisconnect
            };
            var endNotificationRequest = new SendNotificationRequest()
            {
                ClientId = userCode,
                Message = notify,
                Tokens = new List<TokenDTO>() { tokenInfo },
                JsonMessage = Newtonsoft.Json.JsonConvert.SerializeObject(notify),
                NotificationType = notify.NotificationType,
                TransactionType = TransactionTypeEnum.AfterSales,
                RelevanceCode = userCode,
                ReceiverType = ApplicantTypeEnum.Client,
                IsNeedSyn = false,
                WSConnectType = WSConnectTypeEnum.RemoteConnectSecondWindow
            };
            res = _notificationService.PostMessageAsync(endNotificationRequest).Result;
            return res;
        }

        /// <summary>
        /// 远程连接用户不在线逻辑处理
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        private bool RemoteConnectOnlineState(RemoteConnectDTO remoteConnnectInfo, bool isNeedSyn, bool isManangeDisConnnect = false)
        {
            var deviceCode = remoteConnnectInfo.DeviceCode;
            var userToken = remoteConnnectInfo.UserToken;
            var userCode = remoteConnnectInfo.UserCode;
            var deviceToken = remoteConnnectInfo.DeviceToken;
            var key = userToken + "_" + deviceCode;
            var deviceKey = deviceToken + "_" + userCode;
            var roomId = remoteConnnectInfo.RoomId;
            var deviceTokenInfo = CacheMaintenance.Instance.Get<ITokensManager>().Where(x => x.ClientId == deviceCode)?.FirstOrDefault();

            var existRoomId = _remoteConnectManager.GetRoomId(key);
            Logger.WriteLineInfo($"DeviceService RemoteConnectOnlineState key-deviceKey:{key}-{deviceKey}, isNeedSyn: {isNeedSyn}");
            if (string.IsNullOrEmpty(existRoomId))
            {
                Logger.WriteLineWarn($"DeviceService RemoteConnectOnlineState RoomId Empty, key: {key}, isNeedSyn: {isNeedSyn}");
                return true;
            }
            var syncData = new RemoteConnectDataRequest()
            {
                RoomCode = roomId,
                UserRemoteConnect = _remoteConnectManager.GetRemoteConnectDTO(key).MappingTo<UserRemoteConnectRequest>(),
                IsManangeDisConnnect = isManangeDisConnnect
            };
            var closeManagerResult = _remoteConnectManager.Close(key);
            var closeDeviceManagerResult = _remoteConnectManager.DeviceClose(deviceKey);
            var res = false;
            if (isNeedSyn)
            {
                SyncToMasterAsync(SyncTypeEnum.UserRemoteConnectOutline, roomId, syncData).GetAwaiter().GetResult();
            }
            //向设备端发送断开连接申请
            var notification = new ConnectStatusToDeviceNotification()//设备通知
            {
                ControlUserCode = userCode,
                ControlUserName = "",
                ControlType = ControlDeviceParameterEnum.UserTimeout,
                LoginSource = remoteConnnectInfo.LoginSource,
                DeviceCode = deviceCode,
            };
            BroadcastNotificationRequest notificationRequest = new BroadcastNotificationRequest();
            notificationRequest.ClientIds = new List<string>() { deviceCode };
            notificationRequest.Message = notification;
            notificationRequest.JsonMessage = Newtonsoft.Json.JsonConvert.SerializeObject(notification);
            notificationRequest.NotificationType = notification.NotificationType;
            notificationRequest.TransactionType = TransactionTypeEnum.AfterSales;
            notificationRequest.RelevanceCode = roomId;
            notificationRequest.ReceiverType = ApplicantTypeEnum.Device;
            notificationRequest.IsNeedSyn = false;
            notificationRequest.WSConnectType = WSConnectTypeEnum.RemoteConnectSecondWindow;
            if (deviceTokenInfo != null && deviceTokenInfo.IsOldPlatform && deviceTokenInfo.IsOnline)
            {
                var result = _deviceForwardService.ApplyRemoteConnectAsync(notification).Result;
            }
            res = _notificationService.BroadcastMessageAsync(notificationRequest).Result;
            //同时向客户端发送通知，通知断开
            var tokenInfo = _authenticationService.GetTokenAsync(new TokenRequest() { Token = userToken }).Result;
            var notify = new DeviceDisconnectRemoteControlNotification()
            {
                DeviceCode = deviceCode,
                RemoteDeviceState = isManangeDisConnnect ? RemoteDeviceStateEnum.DeviceDisconnect : RemoteDeviceStateEnum.UserTimeOutDisconnect
            };
            var endNotificationRequest = new SendNotificationRequest()
            {
                ClientId = userCode,
                Message = notify,
                Tokens = new List<TokenDTO>() { tokenInfo },
                JsonMessage = Newtonsoft.Json.JsonConvert.SerializeObject(notify),
                NotificationType = notify.NotificationType,
                TransactionType = TransactionTypeEnum.AfterSales,
                RelevanceCode = userCode,
                ReceiverType = ApplicantTypeEnum.Client,
                IsNeedSyn = false,
                WSConnectType = WSConnectTypeEnum.RemoteConnectSecondWindow
            };
            res = _notificationService.PostMessageAsync(endNotificationRequest).Result;
            return res;
        }

        /// <summary>
        /// 远程控制请求接口,包含Get获取初始化配置
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> ApplyRemoteControlAsync(ControlDeviceParameterRequest request)
        {
            var result = false;
            if (!string.IsNullOrEmpty(request.ConsultationCode))
            {
                var liveParameterReq = new WingInterfaceLibrary.LiveConsultation.ControlDeviceParameterInConsultationRequest()
                {
                    Token = request.Token,
                    ConsultationCode = request.ConsultationCode,
                    Parameters = request.Parameters,
                    ControlType = request.ControlType
                };
                result = await _liveConsultationService.ControlParameterAsync(liveParameterReq);
                return result;
            }
            if (string.IsNullOrEmpty(request.DeviceCode))
            {
                ThrowCustomerException(CustomerRpcCode.NoExistDevice, "Device does not exist");
            }
            var userInfo = await _authenticationService.GetTokenAsync(new TokenRequest() { Token = request.Token }) ?? new TokenDTO();
            if (string.IsNullOrEmpty(userInfo.ClientId))
            {
                ThrowCustomerException(CustomerRpcCode.UsercodeIsEmpty, "User code is empty");
            }
            var userCode = userInfo.ClientId;
            //获取房间缓存, 通用
            var isControllingParameter = true;
            if (request.ControlType == ControlDeviceParameterEnum.End)
            {
                isControllingParameter = false;
            }
            // if (existMember.IsControllingParameter != isControllingParameter)
            // {
            //     var changeStateResult = await _rtcService.ChangeControllingStateAsync(new ChangeControllingStateRequest
            //     {
            //         LiveRoomCode = room.LiveRoomCode,
            //         UserCode = userCode,
            //         IsControllingParameter = isControllingParameter,
            //     });
            // }
            if (isControllingParameter)
            {
                var deviceCode = request.DeviceCode;
                var notification = new DeviceControlledParametersNotification()//设备通知
                {
                    ControlUserCode = userCode,
                    ControlUserName = userInfo.AccountName,
                    ControlType = request.ControlType,
                    Parameters = request.Parameters,
                    LoginSource = userInfo.LoginSource,
                    DeviceCode = deviceCode,
                };
                BroadcastNotificationRequest notificationRequest = new BroadcastNotificationRequest();
                notificationRequest.ClientIds = new List<string>() { request.DeviceCode };
                notificationRequest.Message = notification;
                notificationRequest.JsonMessage = Newtonsoft.Json.JsonConvert.SerializeObject(notification);
                notificationRequest.NotificationType = notification.NotificationType;
                notificationRequest.TransactionType = TransactionTypeEnum.AfterSales;
                notificationRequest.RelevanceCode = request.DeviceCode;
                notificationRequest.ReceiverType = ApplicantTypeEnum.Device;
                notificationRequest.WSConnectType = WSConnectTypeEnum.RemoteConnectSecondWindow;
                notificationRequest.IsNeedSyn = true;
                var deviceTokenInfo = CacheMaintenance.Instance.Get<ITokensManager>().Where(x => x.ClientId == deviceCode)?.FirstOrDefault();
                if (deviceTokenInfo != null && deviceTokenInfo.IsOldPlatform && deviceTokenInfo.IsOnline)
                {
                    await _deviceForwardService.DeviceApplyRemoteControlAsync(notification);
                }
                result = await _notificationService.BroadcastMessageAsync(notificationRequest);
            }
            return result;
        }

        /// <summary>
        /// 获取日志请求接口
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> GetRemoteLogAsync(GetRemoteLogRequest request)
        {
            var userInfo = await _authenticationService.GetTokenAsync(new TokenRequest() { Token = request.Token }) ?? new TokenDTO();
            if (string.IsNullOrEmpty(userInfo.ClientId))
            {
                ThrowCustomerException(CustomerRpcCode.UsercodeIsEmpty, "User code is empty");
            }
            var userCode = userInfo.ClientId;
            if (string.IsNullOrEmpty(request.DeviceCode))
            {
                ThrowCustomerException(CustomerRpcCode.NoExistDevice, "Device does not exist");
            }
            var startTime = new DateTime(1970, 1, 1);
            var endTime = DateTime.UtcNow;
            var fileName = string.Empty;
            var startTimeBJ = DateTime.MinValue;
            var endTimeBJ = DateTime.Now.AddMinutes(1);
            switch (request.LogTime)
            {
                case LogTimeEnum.Today:
                    startTimeBJ = DateTime.Now.AddDays(-1);
                    startTime = startTimeBJ.ToUniversalTime();
                    endTime = endTimeBJ.ToUniversalTime();
                    fileName = $"Log_{startTimeBJ.ToString("yyyyMMddTHHmmss")}_{endTimeBJ.ToString("yyyyMMddTHHmmss")}";
                    break;
                case LogTimeEnum.OneWeek:
                    startTimeBJ = DateTime.Now.AddDays(-7);
                    startTime = startTimeBJ.ToUniversalTime();
                    endTime = endTimeBJ.ToUniversalTime();
                    fileName = $"Log_{startTimeBJ.ToString("yyyyMMddTHHmmss")}_{endTimeBJ.ToString("yyyyMMddTHHmmss")}";
                    break;
                case LogTimeEnum.OneMonth:
                    startTimeBJ = DateTime.Now.AddMonths(-1);
                    startTime = startTimeBJ.ToUniversalTime();
                    endTime = endTimeBJ.ToUniversalTime();
                    fileName = $"Log_{startTimeBJ.ToString("yyyyMMddTHHmmss")}_{endTimeBJ.ToString("yyyyMMddTHHmmss")}";
                    break;
                case LogTimeEnum.OneYear:
                    startTimeBJ = DateTime.Now.AddYears(-1);
                    startTime = startTimeBJ.ToUniversalTime();
                    endTime = endTimeBJ.ToUniversalTime();
                    fileName = $"Log_{startTimeBJ.ToString("yyyyMMddTHHmmss")}_{endTimeBJ.ToString("yyyyMMddTHHmmss")}";
                    break;
                default:
                    fileName = $"Log_AllTime";
                    break;
            }
            var deviceCode = request.DeviceCode;
            var key = $"Log_{deviceCode}_{userInfo.ClientId}";
            await AddRemoteMaintainData(key, fileName);
            var notification = new GetRemoteLogToDeviceNotification()//设备通知
            {
                ControlUserCode = userCode,
                StartTime = startTime,
                EndTime = endTime,
                LoginSource = userInfo.LoginSource,
                DeviceCode = deviceCode,
                ControlUserName = userInfo.AccountName,
            };
            BroadcastNotificationRequest notificationRequest = new BroadcastNotificationRequest();
            notificationRequest.ClientIds = new List<string>() { deviceCode };
            notificationRequest.Message = notification;
            notificationRequest.JsonMessage = Newtonsoft.Json.JsonConvert.SerializeObject(notification);
            notificationRequest.NotificationType = notification.NotificationType;
            notificationRequest.TransactionType = TransactionTypeEnum.AfterSales;
            notificationRequest.RelevanceCode = deviceCode;
            notificationRequest.ReceiverType = ApplicantTypeEnum.Device;
            notificationRequest.WSConnectType = WSConnectTypeEnum.RemoteConnectSecondWindow;
            var deviceTokenInfo = CacheMaintenance.Instance.Get<ITokensManager>().Where(x => x.ClientId == deviceCode)?.FirstOrDefault();
            if (deviceTokenInfo != null && deviceTokenInfo.IsOldPlatform && deviceTokenInfo.IsOnline)
            {
                await _deviceForwardService.GetRemoteLogAsync(notification);
            }
            var res = await _notificationService.BroadcastMessageAsync(notificationRequest);
            return res;
        }

        /// <summary>
        /// 设备端响应日志下载地址请求接口
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> ResponseRemoteLogAsync(RemoteLogResponseRequest request)
        {
            Logger.WriteLineWarn($"WingDeviceService-ResponseRemoteLogAsync Init, request:{JsonConvert.SerializeObject(request)}");
            string deviceCode = await GetClientIdByTokenAsync(request.Token);
            if (string.IsNullOrEmpty(deviceCode))
            {
                ThrowCustomerException(CustomerRpcCode.NoExistDevice, "Device does not exist");
            }
            if (string.IsNullOrEmpty(request.UserCode))
            {
                ThrowCustomerException(CustomerRpcCode.UsercodeIsEmpty, "User code is empty");
            }
            var notification = new GetRemoteLogToClientNotification()//设备通知
            {
                DeviceCode = deviceCode,
                LogFileToken = request.LogFileToken,
                Rate = request.Rate,
                RemoteDeviceState = request.RemoteDeviceState
            };
            if (!string.IsNullOrEmpty(request.LogFileToken))
            {
                var key = $"Log_{deviceCode}_{request.UserCode}";
                string fileName = await DeleteRemoteMaintainData(key);
                //var tokenInfo = _authenticationService.GetTokenAsync(new TokenRequest() { Token = userToken }).Result;
                notification = new GetRemoteLogToClientNotification()//设备通知
                {
                    DeviceCode = deviceCode,
                    LogFileToken = request.LogFileToken,
                    Rate = request.Rate,
                    RemoteDeviceState = request.RemoteDeviceState,
                    FileName = fileName
                };
            }
            BroadcastNotificationRequest notificationRequest = new BroadcastNotificationRequest();
            notificationRequest.ClientIds = new List<string>() { request.UserCode };
            notificationRequest.Message = notification;
            //notificationRequest.Tokens = new List<TokenDTO>() {tokenInfo};
            notificationRequest.JsonMessage = Newtonsoft.Json.JsonConvert.SerializeObject(notification);
            notificationRequest.NotificationType = notification.NotificationType;
            notificationRequest.TransactionType = TransactionTypeEnum.AfterSales;
            notificationRequest.RelevanceCode = request.UserCode;
            notificationRequest.ReceiverType = ApplicantTypeEnum.Client;
            notificationRequest.IsNeedSyn = true;
            notificationRequest.WSConnectType = WSConnectTypeEnum.RemoteConnectSecondWindow;
            var res = await _notificationService.BroadcastMessageAsync(notificationRequest);
            Logger.WriteLineWarn($"WingDeviceService-ResponseRemoteLogAsync Last, res:{res}");
            return res;
        }

        /// <summary>
        /// 用户取消日志下载
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> UserCancelLogDownloadAsync(GetDeviceRequest request)
        {
            var userInfo = await _authenticationService.GetTokenAsync(new TokenRequest() { Token = request.Token }) ?? new TokenDTO();
            var userCode = userInfo.ClientId;
            if (string.IsNullOrEmpty(userCode))
            {
                ThrowCustomerException(CustomerRpcCode.UsercodeIsEmpty, "User code is empty");
            }
            var deviceCode = request.DeviceCode;
            if (string.IsNullOrEmpty(deviceCode))
            {
                ThrowCustomerException(CustomerRpcCode.NoExistDevice, "Device does not exist");
            }
            var notification = new CancelLogDownloadNotification()//设备通知
            {
                ControlUserCode = userCode,
                LoginSource = userInfo.LoginSource,
                DeviceCode = deviceCode,
                ControlUserName = userInfo.AccountName,
            };
            BroadcastNotificationRequest notificationRequest = new BroadcastNotificationRequest();
            notificationRequest.ClientIds = new List<string>() { deviceCode };
            notificationRequest.Message = notification;
            notificationRequest.JsonMessage = Newtonsoft.Json.JsonConvert.SerializeObject(notification);
            notificationRequest.NotificationType = notification.NotificationType;
            notificationRequest.TransactionType = TransactionTypeEnum.AfterSales;
            notificationRequest.RelevanceCode = deviceCode;
            notificationRequest.ReceiverType = ApplicantTypeEnum.Device;
            notificationRequest.WSConnectType = WSConnectTypeEnum.RemoteConnectSecondWindow;
            var deviceTokenInfo = CacheMaintenance.Instance.Get<ITokensManager>().Where(x => x.ClientId == deviceCode)?.FirstOrDefault();
            if (deviceTokenInfo != null && deviceTokenInfo.IsOldPlatform && deviceTokenInfo.IsOnline)
            {
                await _deviceForwardService.CancelLogDownloadAsync(notification);
            }
            var res = await _notificationService.BroadcastMessageAsync(notificationRequest);
            if (res)
            {
                var clientNotification = new GetRemoteLogToClientNotification()//客户端通知
                {
                    DeviceCode = request.DeviceCode,
                    LogFileToken = "",
                    Rate = 0,
                    RemoteDeviceState = RemoteDeviceStateEnum.DeviceCancelLogDownload
                };
                notificationRequest = new BroadcastNotificationRequest();
                notificationRequest.ClientIds = new List<string>() { userCode };
                notificationRequest.Message = clientNotification;
                notificationRequest.JsonMessage = Newtonsoft.Json.JsonConvert.SerializeObject(clientNotification);
                notificationRequest.NotificationType = clientNotification.NotificationType;
                notificationRequest.TransactionType = TransactionTypeEnum.AfterSales;
                notificationRequest.RelevanceCode = userCode;
                notificationRequest.ReceiverType = ApplicantTypeEnum.Client;
                res = await _notificationService.BroadcastMessageAsync(notificationRequest);
            }
            return res;
        }

        /// <summary>
        /// 设备取消日志下载
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> DeviceCancelLogDownloadAsync(DeivceCancelLogDownloadRequest request)
        {
            Logger.WriteLineWarn($"WingDeviceService-DeviceCancelLogDownloadAsync Init, request:{JsonConvert.SerializeObject(request)}");
            var req = new RemoteLogResponseRequest()//设备通知客户端
            {
                Token = request.Token,
                LogFileToken = "",
                Rate = 0,
                RemoteDeviceState = RemoteDeviceStateEnum.DeviceCancelLogDownload,
                UserCode = request.UserCode
            };
            var res = await ResponseRemoteLogAsync(req);
            Logger.WriteLineWarn($"WingDeviceService-DeviceCancelLogDownloadAsync Last, res:{res}");
            return res;
        }

        /// <summary>
        /// 查询用户设备连接状态列表
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<IList<DeviceConnectStateResult>> GetDeviceStateListByTokenAsync(GetDeviceStateListRequest request)
        {
            var result = new List<DeviceConnectStateResult>();
            string userCode = await GetClientIdByTokenAsync(request.Token);
            if (string.IsNullOrEmpty(userCode))
            {
                ThrowCustomerException(CustomerRpcCode.UsercodeIsEmpty, "User cod is empty");
            }
            var dataList = _remoteConnectManager.GetAllRemoteConnectByUserToken(request.Token);
            if (dataList?.Count > 0)
            {
                if (request.DeviceCodes?.Count > 0)
                {
                    foreach (var item in request.DeviceCodes)
                    {
                        var resItem = new DeviceConnectStateResult()
                        {
                            DeviceCode = item
                        };
                        var exist = dataList.Find(c => c.DeviceCode == item);
                        if (exist != null)
                        {
                            resItem.ConnectStatus = exist.StatusEnum;
                        }
                        result.Add(resItem);
                    }
                }
                else
                {
                    result = dataList.Select(c => new DeviceConnectStateResult()
                    {
                        DeviceCode = c.DeviceCode,
                        ConnectStatus = c.StatusEnum
                    }).ToList();
                }
            }
            return result;
        }

        /// <summary>
        /// 客户端申请探头应用设置
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> ApplyProbeApplicationSettingAsync(ProbeApplicationSettingRequest request)
        {
            Logger.WriteLineWarn($"WingDeviceService-ProbeApplicationSettingRequest Init, request:{JsonConvert.SerializeObject(request)}");
            var userInfo = await _authenticationService.GetTokenAsync(new TokenRequest() { Token = request.Token }) ?? new TokenDTO();
            if (string.IsNullOrEmpty(userInfo.ClientId))
            {
                ThrowCustomerException(CustomerRpcCode.UsercodeIsEmpty, "User code is empty");
            }
            var userCode = userInfo.ClientId;
            var deviceCode = request.DeviceCode;
            if (string.IsNullOrEmpty(deviceCode))
            {
                ThrowCustomerException(CustomerRpcCode.NoExistDevice, "Device does not exist");
            }
            //添加到缓存，然后在发送通知
            var key = request.DeviceCode;
            var cacheControllingParameter = new CacheControllingParameter()
            {
                Code = userCode + "_" + deviceCode,
                ProbeApplication = JsonConvert.SerializeObject(request.ProbeApplicationSetting)
            };
            await AddProbeApplicationSetting(key, cacheControllingParameter);
            var notification = new ApplyProbeApplicationSettingNotification()//设备通知
            {
                UserCode = userCode,
                ControlType = request.ControlType,
                LoginSource = userInfo.LoginSource,
                DeviceCode = deviceCode,
                ControlUserName = userInfo.AccountName,
            };
            BroadcastNotificationRequest notificationRequest = new BroadcastNotificationRequest();
            notificationRequest.ClientIds = new List<string>() { deviceCode };
            notificationRequest.Message = notification;
            notificationRequest.JsonMessage = Newtonsoft.Json.JsonConvert.SerializeObject(notification);
            notificationRequest.NotificationType = notification.NotificationType;
            notificationRequest.TransactionType = TransactionTypeEnum.AfterSales;
            notificationRequest.RelevanceCode = deviceCode;
            notificationRequest.ReceiverType = ApplicantTypeEnum.Device;
            notificationRequest.WSConnectType = WSConnectTypeEnum.RemoteConnectSecondWindow;
            var deviceTokenInfo = CacheMaintenance.Instance.Get<ITokensManager>().Where(x => x.ClientId == deviceCode)?.FirstOrDefault();
            if (deviceTokenInfo != null && deviceTokenInfo.IsOldPlatform && deviceTokenInfo.IsOnline)
            {
                await _deviceForwardService.ApplyProbeApplicationSettingAsync(notification);
            }
            var res = await _notificationService.BroadcastMessageAsync(notificationRequest);
            Logger.WriteLineWarn($"WingDeviceService-ProbeApplicationSettingRequest Last, res:{res}");
            return res;
        }

        /// <summary>
        /// 设备端响应探头应用设置
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> ResponseProbeApplicationSettingAsync(ProbeApplicationSettingResultRequest request)
        {
            Logger.WriteLineWarn($"WingDeviceService-ResponseProbeApplicationSettingAsync Init, request:{JsonConvert.SerializeObject(request)}");
            string deviceCode = await GetClientIdByTokenAsync(request.Token);
            if (string.IsNullOrEmpty(request.UserCode))
            {
                ThrowCustomerException(CustomerRpcCode.UsercodeIsEmpty, "User code is empty");
            }
            if (string.IsNullOrEmpty(deviceCode))
            {
                ThrowCustomerException(CustomerRpcCode.NoExistDevice, "Device does not exist");
            }
            //添加到缓存，然后在发送通知
            var key = deviceCode;
            var cacheControllingParameter = new CacheControllingParameter()
            {
                Code = request.UserCode + "_" + deviceCode,
                ProbeApplication = JsonConvert.SerializeObject(request.ProbeApplicationSetting)
            };
            await AddProbeApplicationSetting(key, cacheControllingParameter);

            var notification = new ProbeApplicationSettingResponseNotification()//客户端通知
            {
                DeviceCode = deviceCode,
                ControlType = request.ControlType
            };
            BroadcastNotificationRequest notificationRequest = new BroadcastNotificationRequest();
            notificationRequest.ClientIds = new List<string>() { request.UserCode };
            notificationRequest.Message = notification;
            notificationRequest.JsonMessage = Newtonsoft.Json.JsonConvert.SerializeObject(notification);
            notificationRequest.NotificationType = notification.NotificationType;
            notificationRequest.TransactionType = TransactionTypeEnum.AfterSales;
            notificationRequest.RelevanceCode = request.UserCode;
            notificationRequest.ReceiverType = ApplicantTypeEnum.Client;
            notificationRequest.IsNeedSyn = true;
            notificationRequest.WSConnectType = WSConnectTypeEnum.RemoteConnectSecondWindow;
            var res = await _notificationService.BroadcastMessageAsync(notificationRequest);
            var notify = new DeviceParametersNotification() //调参
            {
                DeviceCode = deviceCode,
                IsResponse = true
            };
            BroadcastNotificationRequest notifyRequest = new BroadcastNotificationRequest();
            notifyRequest.ClientIds = new List<string>() { request.UserCode };
            notifyRequest.Message = notify;
            notifyRequest.JsonMessage = Newtonsoft.Json.JsonConvert.SerializeObject(notify);
            notifyRequest.NotificationType = notify.NotificationType;
            notifyRequest.TransactionType = TransactionTypeEnum.AfterSales;
            notifyRequest.RelevanceCode = request.UserCode;
            notifyRequest.ReceiverType = ApplicantTypeEnum.Client;
            notifyRequest.IsNeedSyn = true;
            notifyRequest.WSConnectType = WSConnectTypeEnum.RemoteConnectSecondWindow;
            res = await _notificationService.BroadcastMessageAsync(notifyRequest);


            Logger.WriteLineWarn($"WingDeviceService-ResponseProbeApplicationSettingAsync Last, res:{res}, notification: {deviceCode}");
            return res;
        }

        /// <summary>
        /// 客户端/设备端获取探头应用数据请求
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<ProbeApplicationSettingInfoDTO> GetProbeApplicationSettingAsync(GetControlParametersRequest request)
        {
            Logger.WriteLineWarn($"WingDeviceService-GetProbeApplicationSettingAsync Init, request:{JsonConvert.SerializeObject(request)}");
            var result = new ProbeApplicationSettingInfoDTO();
            var deviceCode = request.DeviceCode;
            //检查设备编码不为空
            if (string.IsNullOrWhiteSpace(deviceCode))
            {
                ThrowCustomerException(CustomerRpcCode.DeviceCodeIsEmpty, "Required parameter:DeviceCode cannot be empty");
            }
            var controlParameterData = await GetProbeApplicationSetting(deviceCode);
            if (controlParameterData != null && !string.IsNullOrEmpty(controlParameterData.ProbeApplication))
            {
                result = JsonConvert.DeserializeObject<ProbeApplicationSettingInfoDTO>(controlParameterData.ProbeApplication);
            }
            Logger.WriteLineWarn($"WingDeviceService-GetProbeApplicationSettingAsync Last, ProbeApplication:{controlParameterData.ProbeApplication}");
            return result;
        }

        /// <summary>
        /// 发送指令给设备
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<string> SendCommandToDeviceAsync(SendCommandToDeviceRequest request)
        {
            if (request == null || string.IsNullOrWhiteSpace(request.DeviceCode))
            {
                ThrowCustomerException(CustomerRpcCode.NotificationParamError, "NotificationParamError");
            }
            string userCode = await GetClientIdByTokenAsync(request.Token);
            var resultCode = Guid.NewGuid().ToString("N");
            var notify = new SendCommandToDeviceNotification
            {
                ActionType = request.ActionType,
                ResultCode = resultCode,
                Sender = userCode,
                DeviceCode = request.DeviceCode,
                OperatorCode = userCode,
            };
            var json = JsonConvert.SerializeObject(request);
            await AddRemoteMaintainData(resultCode, json);
            var commonRequest = new TransportAfterSalesCommonRequest
            {
                NotificationType = notify.NotificationType,
                ReceiverId = request.DeviceCode,
                ReqJson = JsonConvert.SerializeObject(notify)
            };
            var deviceTokenInfo = CacheMaintenance.Instance.Get<ITokensManager>().Where(x => x.ClientId == request.DeviceCode)?.FirstOrDefault();
            if (deviceTokenInfo != null && deviceTokenInfo.IsOldPlatform && deviceTokenInfo.IsOnline)
            {
                await _deviceForwardService.SendCommandToDeviceAsync(notify);
            }
            if (await TransportAfterSalesCommonAsync(notify, commonRequest))
            {
                return resultCode;
            }
            return string.Empty;
        }

        /// <summary>
        /// 发送结果给客户端
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<string> SendResultToClientAsync(SendResultToClientRequest request)
        {
            if (request == null || string.IsNullOrWhiteSpace(request.UserCode))
            {
                ThrowCustomerException(CustomerRpcCode.NotificationParamError, "NotificationParamError");
            }
            string userCode = await GetClientIdByTokenAsync(request.Token);
            var resultCode = Guid.NewGuid().ToString("N");
            
            var json = JsonConvert.SerializeObject(request);
            await AddRemoteMaintainData(resultCode, json);
            if (string.IsNullOrEmpty(request.Settings) || request.Settings.ToLower().Equals("null"))
            {
                var notify = new SystemSettingSaveAndExitToClientNotification
                {
                    ResultCode = resultCode,
                    Sender = userCode
                };
                
                var commonRequest = new TransportAfterSalesCommonRequest
                {
                    NotificationType = notify.NotificationType,
                    ReceiverId = request.UserCode,
                    ReqJson = JsonConvert.SerializeObject(notify)
                };
                if (await TransportAfterSalesCommonAsync(notify, commonRequest))
                {
                    return resultCode;
                }
            }
            else
            {
                var notify = new SendResultToClientNotification
                {
                    ResultCode = resultCode,
                    Sender = userCode
                };
                var commonRequest = new TransportAfterSalesCommonRequest
                {
                    NotificationType = notify.NotificationType,
                    ReceiverId = request.UserCode,
                    ReqJson = JsonConvert.SerializeObject(notify)
                };
                if (await TransportAfterSalesCommonAsync(notify, commonRequest))
                {
                    return resultCode;
                }
            }   
            
            return string.Empty;
        }

        /// <summary>
        /// 从Server获取远程维护结果
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<string> GetResultFromServerAsync(GetResultFromServerRequest request)
        {
            if (request == null || string.IsNullOrWhiteSpace(request.ResultCode))
            {
                ThrowCustomerException(CustomerRpcCode.CodeIsEmpty, "CodeIsEmpty");
            }
            string userCode = await GetClientIdByTokenAsync(request.Token);
            var resultJson = await DeleteRemoteMaintainData(request.ResultCode);
            return resultJson;
        }

        /// <summary>
        /// 通用的发送请求方法
        /// </summary>
        /// <param name="request">请求参数</param>
        /// <returns>是否发送成功</returns>
        private async Task<bool> TransportAfterSalesCommonAsync(object notification, TransportAfterSalesCommonRequest request)
        {
            BroadcastNotificationRequest notificationRequest = new BroadcastNotificationRequest();
            notificationRequest.ClientIds = new List<string>() { request.ReceiverId };
            notificationRequest.Message = notification;
            notificationRequest.JsonMessage = Newtonsoft.Json.JsonConvert.SerializeObject(notification);
            notificationRequest.NotificationType = request.NotificationType;
            notificationRequest.TransactionType = TransactionTypeEnum.AfterSales;
            notificationRequest.ReceiverType = ApplicantTypeEnum.Device;
            return await _notificationService.BroadcastMessageAsync(notificationRequest);
        }

        /// <summary>
        /// 重启设备
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        /// <errorCodes>DeviceCodeRequired,DeviceNotExist</errorCodes>
        public async Task<bool> RestartDeviceAsync(RestartDeviceRequest request)
        {
            string userCode = await GetClientIdByTokenAsync(request.Token);
            if (string.IsNullOrEmpty(userCode))
            {
                ThrowCustomerException(CustomerRpcCode.UsercodeIsEmpty, "User code is empty");
            }
            if (string.IsNullOrEmpty(request.DeviceCode))
            {
                ThrowCustomerException(CustomerRpcCode.DeviceCodeEmpty, "Device code is empty");
            }
            var deviceCode = request.DeviceCode;
            var deviceInfo = CacheMaintenance.Instance.Get<IDeviceInfosManager>().Get(deviceCode);
            if (deviceInfo == null)
            {
                ThrowCustomerException(CustomerRpcCode.DeviceCodeEmpty, "Device code is empty");
            }
            var deviceTokenInfo = CacheMaintenance.Instance.Get<ITokensManager>().Where(x => x.ClientId == deviceCode)?.FirstOrDefault();
            if (deviceTokenInfo == null && !deviceTokenInfo.IsOnline)
            {
                ThrowCustomerException(CustomerRpcCode.DeviceNotOnline, "Device Not Online");
            }
            if (deviceInfo.DeviceType.ToLower() != "sonopost" || (deviceTokenInfo != null && deviceTokenInfo.IsOldPlatform))
            {
                ThrowCustomerException(CustomerRpcCode.DeviceCannotRestart, "Device Cannot Restart");
            }

            //var userTokenInfo = _authenticationService.GetTokenAsync(new TokenRequest() { Token = request.Token }).Result;
            var notification = new RestartDeviceNotification()//设备通知
            {
                ControlUserCode = userCode,
                DeviceCode = deviceCode,
            };
            BroadcastNotificationRequest notificationRequest = new BroadcastNotificationRequest();
            notificationRequest.ClientIds = new List<string>() { request.DeviceCode };
            notificationRequest.Message = notification;
            //notificationRequest.Tokens = new List<TokenDTO>() {userTokenInfo};
            notificationRequest.JsonMessage = Newtonsoft.Json.JsonConvert.SerializeObject(notification);
            notificationRequest.NotificationType = notification.NotificationType;
            notificationRequest.TransactionType = TransactionTypeEnum.AfterSales;
            notificationRequest.RelevanceCode = request.DeviceCode;
            notificationRequest.ReceiverType = ApplicantTypeEnum.Device;
            notificationRequest.IsNeedSyn = false;
            notificationRequest.WSConnectType = WSConnectTypeEnum.RemoteConnectSecondWindow;

            if (deviceTokenInfo.IsOldPlatform)
            {
                if (request.IsNeedSyn)
                {
                    //老平台设备请求转发
                    await _deviceForwardService.RestartDeviceAsync(notification);
                }
            }
            else
            {
                await _notificationService.BroadcastMessageAsync(notificationRequest);
            }
            //重启通知发出后，等待设备端调用断开云连接的操作，然后在响应客户端
            await Task.Delay(1000);
            //关闭在线状态
            var key = request.Token + "_" + request.DeviceCode;
            var req = new GetTokensWithClientIdRequest()
            {
                ClientId = request.DeviceCode
            };
            var deviceKey = deviceTokenInfo.Code + "_" + userCode;
            //也要同步通知其他服务器移除
            var syncData = new RemoteConnectDataRequest()
            {
                RoomCode = "",
                UserRemoteConnect = _remoteConnectManager.GetRemoteConnectDTO(key).MappingTo<UserRemoteConnectRequest>(),
            };
            _remoteConnectManager.Close(key);
            _remoteConnectManager.DeviceClose(deviceKey);
            if (request.IsNeedSyn)
            {
                SyncToMasterAsync(SyncTypeEnum.RestartDevice, "", syncData).GetAwaiter().GetResult();
            }
            return true;
        }
        #endregion 

        /// <summary>
        /// 获取设备连接列表
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> ExistConnectingDeviceInfoAsync(GetDeviceRequest request)
        {
            if (string.IsNullOrEmpty(request.DeviceCode))
            {
                ThrowCustomerException(CustomerRpcCode.DeviceCodeIsEmpty, "Device code is empty");
            }
            //根据deviceCode 获取所有的  改设备的链接状态
            var deviceRemoteConnectInfo = _remoteConnectManager.GetDeviceRemoteConnectInfo(request.DeviceCode);
            if (deviceRemoteConnectInfo == null)
            {
                return false;
            }
            if (request.ConnectStatus != null && request.ConnectStatus.Any())
            {
                return request.ConnectStatus.Contains(deviceRemoteConnectInfo.StatusEnum);
            }
            return true;
        }

        /// <summary>
        /// 获取设备连接列表
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<RemoteConnectsRequest> FindConnectingDeviceListAsync(GetDeviceRequest request)
        {
            var result = new RemoteConnectsRequest();
            string userCode = await GetClientIdByTokenAsync(request.Token);
            if (string.IsNullOrEmpty(userCode))
            {
                ThrowCustomerException(CustomerRpcCode.UsercodeIsEmpty, "User code is empty");
            }
            if (string.IsNullOrEmpty(request.DeviceCode))
            {
                ThrowCustomerException(CustomerRpcCode.DeviceCodeIsEmpty, "Device code is empty");
            }
            //根据deviceCode 获取所有的  改设备的链接状态
            var customerRemoteConnectList = _remoteConnectManager.GetAllRemoteConnectList(request.DeviceCode);
            var deviceRemoteConnectList = _remoteConnectManager.GetAllDeviceRemoteConnectList(request.DeviceCode);
            if (customerRemoteConnectList?.Count > 0)
            {
                result.UserRemoteConnect = customerRemoteConnectList.MappingTo<List<UserRemoteConnectRequest>>();
            }
            if (deviceRemoteConnectList?.Count > 0)
            {
                result.DeviceRemoteConnect = deviceRemoteConnectList.MappingTo<List<DeviceRemoteConnectRequest>>();
            }
            return result;
        }

        /// <summary>
        /// 取消设备连接列表
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> CloseConnectingDeviceListAsync(GetDeviceRequest request)
        {
            Logger.WriteLineWarn($"DeviceService CloseConnectingDeviceListAsync Init, DeviceCode: {request.DeviceCode}, isNeedSyn: {request.IsNeedSyn}");
            string uCode = await GetClientIdByTokenAsync(request.Token);
            if (string.IsNullOrEmpty(uCode))
            {
                ThrowCustomerException(CustomerRpcCode.UsercodeIsEmpty, "User code is empty");
            }
            if (string.IsNullOrEmpty(request.DeviceCode))
            {
                ThrowCustomerException(CustomerRpcCode.DeviceCodeIsEmpty, "Device code is empty");
            }
            //根据deviceCode 获取所有的  改设备的链接状态, 并关闭
            var customerRemoteConnectList = _remoteConnectManager.GetAllRemoteConnectList(request.DeviceCode);
            if (customerRemoteConnectList?.Count > 0)
            {
                var deviceTokenInfo = CacheMaintenance.Instance.Get<ITokensManager>().Where(x => x.ClientId == request.DeviceCode)?.FirstOrDefault();
                foreach (var item in customerRemoteConnectList)
                {
                    // var key = item.UserToken + "_" + item.DeviceCode;
                    // _remoteConnectManager.Close(key);

                    var userCode = item.UserCode;
                    var deviceCode = item.DeviceCode;
                    var userToken = item.UserToken;
                    var deviceToken = item.DeviceToken;
                    var key = userToken + "_" + deviceCode;
                    var deviceKey = deviceToken + "_" + userCode;
                    var roomId = item.RoomId;

                    var existRoomId = _remoteConnectManager.GetRoomId(key);
                    if (string.IsNullOrEmpty(existRoomId))
                    {
                        Logger.WriteLineWarn($"DeviceService RemoteConnectOnlineState RoomId Empty, key: {key}, isNeedSyn: {request.IsNeedSyn}");
                        return true;
                    }
                    var syncData = new RemoteConnectDataRequest()
                    {
                        RoomCode = roomId,
                        UserRemoteConnect = _remoteConnectManager.GetRemoteConnectDTO(key).MappingTo<UserRemoteConnectRequest>(),
                        IsManangeDisConnnect = true
                    };
                    var closeManagerResult = _remoteConnectManager.Close(key);
                    var closeDeviceManagerResult = _remoteConnectManager.DeviceClose(deviceKey);
                    var res = false;
                    if (request.IsNeedSyn)
                    {
                        SyncToMasterAsync(SyncTypeEnum.UserRemoteConnectOutline, roomId, syncData).GetAwaiter().GetResult();
                    }
                    //向设备端发送断开连接申请
                    var notification = new ConnectStatusToDeviceNotification()//设备通知
                    {
                        ControlUserCode = userCode,
                        ControlUserName = "",
                        ControlType = ControlDeviceParameterEnum.UserTimeout,
                        LoginSource = item.LoginSource,
                        DeviceCode = deviceCode,
                    };
                    BroadcastNotificationRequest notificationRequest = new BroadcastNotificationRequest();
                    notificationRequest.ClientIds = new List<string>() { deviceCode };
                    notificationRequest.Message = notification;
                    notificationRequest.JsonMessage = Newtonsoft.Json.JsonConvert.SerializeObject(notification);
                    notificationRequest.NotificationType = notification.NotificationType;
                    notificationRequest.TransactionType = TransactionTypeEnum.AfterSales;
                    notificationRequest.RelevanceCode = roomId;
                    notificationRequest.ReceiverType = ApplicantTypeEnum.Device;
                    notificationRequest.IsNeedSyn = false;
                    notificationRequest.WSConnectType = WSConnectTypeEnum.RemoteConnectSecondWindow;
                    if (deviceTokenInfo != null && deviceTokenInfo.IsOldPlatform && deviceTokenInfo.IsOnline)
                    {
                        await _deviceForwardService.ApplyRemoteConnectAsync(notification);
                    }
                    res = _notificationService.BroadcastMessageAsync(notificationRequest).Result;
                    //同时向客户端发送通知，通知断开
                    var tokenInfo = _authenticationService.GetTokenAsync(new TokenRequest() { Token = userToken }).Result;
                    var notify = new DeviceDisconnectRemoteControlNotification()
                    {
                        DeviceCode = deviceCode,
                        RemoteDeviceState = RemoteDeviceStateEnum.DeviceDisconnect
                    };
                    var endNotificationRequest = new SendNotificationRequest()
                    {
                        ClientId = userCode,
                        Message = notify,
                        Tokens = new List<TokenDTO>() { tokenInfo },
                        JsonMessage = Newtonsoft.Json.JsonConvert.SerializeObject(notify),
                        NotificationType = notify.NotificationType,
                        TransactionType = TransactionTypeEnum.AfterSales,
                        RelevanceCode = userCode,
                        ReceiverType = ApplicantTypeEnum.Client,
                        IsNeedSyn = false,
                        WSConnectType = WSConnectTypeEnum.RemoteConnectSecondWindow
                    };
                    res = _notificationService.PostMessageAsync(endNotificationRequest).Result;
                }
            }
            var deviceRemoteConnectList = _remoteConnectManager.GetAllDeviceRemoteConnectList(request.DeviceCode);
            if (deviceRemoteConnectList?.Count > 0)
            {
                foreach (var item in deviceRemoteConnectList)
                {
                    var deviceKey = item.DeviceToken + "_" + item.UserCode;
                    var userCode = item.UserCode;
                    var existRoomId = _remoteConnectManager.GetExistDeviceRoomId(deviceKey);
                    if (!existRoomId)
                    {
                        Logger.WriteLineWarn($"DeviceService CloseConnectingDeviceListAsync Device RoomId Empty, deviceKey: {deviceKey}, isNeedSyn: true");
                        return true;
                    }
                    var roomId = userCode + request.DeviceCode;

                    var userToken = item.UserToken;
                    var deviceCode = item.DeviceCode;
                    var deviceToken = item.DeviceToken;
                    var key = userToken + "_" + deviceCode;
                    var syncData = new RemoteConnectDataRequest()
                    {
                        RoomCode = roomId,
                        UserRemoteConnect = _remoteConnectManager.GetRemoteConnectDTO(key).MappingTo<UserRemoteConnectRequest>(),
                        IsManangeDisConnnect = true
                    };
                    var closeManagerResult = _remoteConnectManager.Close(key);
                    var closeDeviceManagerResult = _remoteConnectManager.DeviceClose(deviceKey);
                    if (request.IsNeedSyn)
                    {
                        SyncToMasterAsync(SyncTypeEnum.DeviceRemoteConnectOutline, roomId, syncData).GetAwaiter().GetResult();
                    }
                    var tokenInfo = _authenticationService.GetTokenAsync(new TokenRequest() { Token = userToken }).Result;
                    var notify = new DeviceDisconnectRemoteControlNotification()
                    {
                        DeviceCode = deviceCode,
                        RemoteDeviceState = RemoteDeviceStateEnum.DeviceDisconnect
                    };
                    var endNotificationRequest = new SendNotificationRequest()
                    {
                        ClientId = userCode,
                        Message = notify,
                        Tokens = new List<TokenDTO>() { tokenInfo },
                        JsonMessage = Newtonsoft.Json.JsonConvert.SerializeObject(notify),
                        NotificationType = notify.NotificationType,
                        TransactionType = TransactionTypeEnum.AfterSales,
                        RelevanceCode = userCode,
                        ReceiverType = ApplicantTypeEnum.Client,
                        IsNeedSyn = false,
                        WSConnectType = WSConnectTypeEnum.RemoteConnectSecondWindow
                    };
                    var res = _notificationService.PostMessageAsync(endNotificationRequest).Result;
                }
            }
            Logger.WriteLineWarn($"DeviceService CloseConnectingDeviceListAsync Completed, DeviceCode: {request.DeviceCode}");
            return true;
        }

        /// <summary>
        /// 添加系统设置临时数据
        /// </summary>
        /// <param name="key">键</param>
        /// <param name="value">值</param>
        /// <returns></returns>
        private async Task AddRemoteMaintainData(string key, string value)
        {
            _remoteMaintainMsgs.AddOrUpdate(key, value, (a, b) => value);
            if (ConfigurationManager.IsDistributed)
            {
                await SyncRemoteDataToMasterAsync(0, key, JsonConvert.SerializeObject(value));
            }
        }

        /// <summary>
        /// 获取并删除系统设置临时数据
        /// </summary>
        /// <param name="key">键</param>
        /// <param name="onSyncCompleted">后续操作</param>
        /// <returns></returns>
        private async Task<string> DeleteRemoteMaintainData(string key, Action onSyncCompleted = null)
        {
            if (_remoteMaintainMsgs.TryRemove(key, out string value))
            {
                if (ConfigurationManager.IsDistributed)
                {
                    if (onSyncCompleted == null) onSyncCompleted = () => { };
                    SyncRemoteDataToMasterAsync(1, key, JsonConvert.SerializeObject(string.Empty)).ConfigureAwait(false).GetAwaiter().OnCompleted(onSyncCompleted);
                }
            }
            return await Task.FromResult<string>(value);
        }

        /// <summary>
        /// 添加探头和应用数据
        /// </summary>
        /// <param name="key">键</param>
        /// <param name="value">值</param>
        /// <returns></returns>
        private async Task AddProbeApplicationSetting(string key, CacheControllingParameter value)
        {
            _cacheProbeApplicationSetting.AddOrUpdate(key, value, (a, b) => value);
            if (ConfigurationManager.IsDistributed)
            {
                await SyncProbeApplicationToMasterAsync(0, key, value);
            }
        }

        /// <summary>
        /// 获取探头和应用数据
        /// </summary>
        /// <param name="key">键</param>
        /// <returns></returns>
        private async Task<CacheControllingParameter> GetProbeApplicationSetting(string key)
        {
            _cacheProbeApplicationSetting.TryGetValue(key, out CacheControllingParameter value);
            return await Task.FromResult<CacheControllingParameter>(value);
        }
    }
}